'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = function (ComponentStyle) {
  var createStyledComponent = function createStyledComponent(target, rules, props) {
    var prevProps = target && typeof target !== 'string' ? (typeof target === 'undefined' ? 'undefined' : _typeof(target)) === 'object' ? target.props : typeof target === 'function' ? target.options.props : {} : {};
    var mergedProps = Object.assign({}, prevProps, props);

    var componentStyle = new ComponentStyle(rules);

    var StyledComponent = {
      props: mergedProps,
      render: function render(createElement) {
        var _this = this;

        return createElement(target, {
          class: [this.generatedClassName],
          props: this.$props,
          domProps: {
            value: this.value
          },
          on: {
            input: function input(event) {
              _this.$emit('input', event.target.value);
            },
            click: function click(event) {
              _this.$emit('click', event);
            }
          }
        }, this.$slots.default);
      },
      methods: {
        generateAndInjectStyles: function generateAndInjectStyles(style, props) {
          return style.generateAndInjectStyles(props);
        }
      },
      computed: {
        generatedClassName: function generatedClassName() {
          var componentProps = Object.assign({}, this.$props);
          return this.generateAndInjectStyles(componentStyle, componentProps);
        }
      },
      extend: function extend() {
        for (var _len = arguments.length, extendedRules = Array(_len), _key = 0; _key < _len; _key++) {
          extendedRules[_key] = arguments[_key];
        }

        var extended = [];

        extendedRules[0].forEach(function (line, key) {
          extended.push(line);
          extended.push(extendedRules[key + 1]);
        });

        return createStyledComponent(target, rules.slice().concat(extended), mergedProps);
      },
      withComponent: function withComponent(newTarget) {
        return createStyledComponent(newTarget, rules, mergedProps);
      }
    };

    return StyledComponent;
  };

  return createStyledComponent;
};

module.exports = exports['default'];